<?php declare(strict_types=1);

namespace App\Models;

use App\Models\Enums\SystemRoleEnum;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Relations\hasMany;
use Illuminate\Database\Eloquent\SoftDeletes;

final class Role extends Model
{
    use HasFactory, SoftDeletes;

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'name',
        'slug'
    ];

    public function scopeLatest(Builder $query): Builder
    {
        return $query->orderBy('id', 'desc');
    }

    public function scopeAlphavit(Builder $query): Builder
    {
        return $query->orderBy('name', 'asc');
    }

    public function permissions(): hasMany
    {
        return $this->hasMany(RolePermission::class, 'role_id', 'id');
    }


    /**
     * Проверяем можем мы удалять эту группу или нет.
     * Есть системные группы, которые нельзя удалять.
     */
    public function isRemove(): bool
    {
        $dontRemove = SystemRoleEnum::toArray();
        return (array_search($this->slug, $dontRemove) === false);
    }

    /**
     * Проверка эта группа самая главная или нет.
     */
    public function isAdmin(): bool
    {
        return ($this->slug === SystemRoleEnum::Admin->value);
    }
}
